/********************************************************************************
    videotrace - show video stream header dumps
 ********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include <libvideocoding/fileio/inputstream_istr.hh>
#include <libvideocoding/mpeg2/syntax/pessource.hh>
#include <libvideocoding/mpeg2/syntax/demux.hh>
#include <libvideocoding/mpeg2/syntax/scsplitter.hh>
#include <libvideocoding/mpeg2/syntax/video.hh>
#include <libvideogfx/utility/bitstream/bitreader.hh>

#include <unistd.h>
#include <stdio.h>
#include <iostream>
#include <iomanip>
#include <fstream>
using namespace std;
using namespace videocoding;

int main(int argc,char** argv)
{
  try
    {
      ifstream instr(argv[1]);

      InputStream_IStream istr;
      istr.SetIStream(instr);

      MP2_UniversalStream_Decoder progdec;
      progdec.SetInputStream(istr);
      //progdec.ForceStreamType(MPEG_Program_Stream);

      MP2_PES_Demux demux;
      demux.SetPESSource(progdec);
      demux.EnableChannel(MP2_StartCode_Video0);

      MP2_PES_DemuxExtractor videoextr;
      videoextr.SetSource(demux);
      videoextr.SetChannel(MP2_StartCode_Video0);

      MP2_StartCodeSplitter splitter;
      splitter.SetSource(videoextr);
      splitter.SetBreakpointAllSC();
      splitter.SetBreakpoint(MP2_StartCode_ExtensionStart,false);

      MP2_PES_JunkFilter junkfilter;
      junkfilter.SetSource(splitter);

      MP2_SequenceHeader seqhdr;
      MP2_PictureHeader pichdr;

      pichdr.seqhdr = &seqhdr;

      for (;;)
	{
	  MP2_SystemPacket* pck = junkfilter.GetNextPacket();
	  if (pck==NULL)
	    break;

	  MP2_PES_Packet* pes = dynamic_cast<MP2_PES_Packet*>(pck);
	  Assert(pes);

	  BitReader br(pes->data.AskData(), pes->data.AskLength());

	  MP2_StartCode sc = (MP2_StartCode)(br.PeekBits(32) & 0xFF);

	  switch(sc)
	    {
	    case MP2_StartCode_SequenceHeader:
	      {
		seqhdr.Read(br);

		cout << seqhdr << endl;
	      }
	      break;

	    case MP2_StartCode_GroupStart:
	      {
		MP2_GOPHeader hdr;
		hdr.Read(br);

		cout << hdr << endl;
	      }
	      break;

	    case MP2_StartCode_PictureStart:
	      {
		pichdr.Read(br);

		cout << pichdr << endl;
	      }
	      break;
	    }

	  if (IsMPEG_SLICE_START(sc))
	    {
	      MP2_SliceHeader hdr;
	      hdr.pichdr = &pichdr;
	      hdr.Read(br);

	      cout << hdr << endl;
	    }

	  delete pes;
	}
    }
  catch (const Excpt_Base& e)
    {
      MessageDisplay::Show(e);
    }

  return 0;
}
