/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/dct/idct_mmx.hh"
#include <libvideogfx/types.hh>
#include <algorithm>

extern "C" {
extern void idct_mmx( videogfx::int16 * blk );
}

namespace videocoding {
  using namespace videogfx;

  void IDCTBlk_MMX8::CalcIDCT(Pixel* out,int out_stride,const DCTBlock_8x8& blk,bool add)
  {
    int16 inbuf[8*8];
    int16* p=inbuf;

    for (int v=0;v<8;v++)
      for (int u=0;u<8;u++)
	{
	  *p++ = blk.coeff[v][u];
	}

    idct_mmx(inbuf);

    if (add)
      {
#if 0
	for (int y=0;y<8;y++)
	  for (int x=0;x<8;x++)
	    {
	      int v = inbuf[y*8+x] + out[y*out_stride+x];
	      if (v<0) v=0; else if (v>255) v=255;
	      out[y*out_stride+x] = v;
	    }
#else
	__asm__
	  (
	   "pxor %mm0, %mm0\n\t"
	   );

	for (int y=0;y<8;y++)
	  {
	    void* inp = &inbuf[y*8];
	    void* outp = &out[y*out_stride];

	    __asm__
	      (
	       "movq      (%0), %%mm1\n\t"
	       "movq     8(%0), %%mm2\n\t"
	       "movq      (%1), %%mm3\n\t"
	       "movq     %%mm3, %%mm4\n\t"

	       "punpcklbw %%mm0, %%mm3\n\t"
	       "punpckhbw %%mm0, %%mm4\n\t"

	       "paddw     %%mm3, %%mm1\n\t"
	       "paddw     %%mm4, %%mm2\n\t"

	       "packuswb %%mm2, %%mm1\n\t"
	       "movq     %%mm1, (%1)\n\t"

	       : : "r"(inp),"r"(outp)
	       );
	  }
#endif
      }
    else
      {
#if 0
	for (int y=0;y<8;y++)
	  for (int x=0;x<8;x++)
	    {
	      int v = inbuf[y*8+x];
	      if (v<0) v=0; else if (v>255) v=255;
	      out[y*out_stride+x] = v;
	    }
#else
	for (int y=0;y<8;y++)
	  {
	    void* inp = &inbuf[y*8];
	    void* outp = &out[y*out_stride];

	    __asm__
	      (
	       "movq      (%0), %%mm1\n\t"
	       "movq     8(%0), %%mm2\n\t"
	       "packuswb %%mm2, %%mm1\n\t"
	       "movq     %%mm1, (%1)\n\t"

	       : : "r"(inp),"r"(outp)
	       );

	    //movq_mi2r(inp

	    //if (v<0) v=0; else if (v>255) v=255;
	    //out[y*out_stride+x] = v;
	  }
#endif
      }
  }

}
