/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/motioncomp/motioncomp_mmx2.hh"

namespace videocoding {

  void MotionCompensation_MMX2_8::Compensate(Pixel* dest,int dest_stride,
					     const Bitmap<Pixel>& refbm,
					     int ref_x0,int ref_y0,
					     const MotionVector& mv,int blkh)
  {
    int mv_h = mv.h>>16;
    int mv_v = mv.v>>16;

    bool half_h = (mv.h & 0x8000);
    bool half_v = (mv.v & 0x8000);

    const Pixel*const* ref = refbm.AskFrame();

    int  src_stride = refbm.AskStride();
    const Pixel* sp = &ref[ref_y0+mv_v][ref_x0+mv_h];
    Pixel* dp = dest;

    if (!half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    const uint64* spd = (const uint64*)sp;
	    uint64* dpd = (uint64*)dp;

	    *dpd = *spd;

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "movq  %%mm1,(%1)\n\t"

	       : : "r"(sp),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if (!half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "movq  %%mm1,(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "movq   (%1),%%mm2\n\t"
	       "pavgb 1(%1),%%mm2\n\t"
	       "pavgb %%mm1,%%mm2\n\t"
	       "movq  %%mm2,(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
  }



  void MotionCompensation_MMX2_8avg::Compensate(Pixel* dest,int dest_stride,
						const Bitmap<Pixel>& refbm,
						int ref_x0,int ref_y0,
						const MotionVector& mv,int blkh)
  {
    int mv_h = mv.h>>16;
    int mv_v = mv.v>>16;

    bool half_h = (mv.h & 0x8000);
    bool half_v = (mv.v & 0x8000);

    const Pixel*const* ref = refbm.AskFrame();

    int  src_stride = refbm.AskStride();
    const Pixel* sp = &ref[ref_y0+mv_v][ref_x0+mv_h];
    Pixel* dp = dest;

    if (!half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "movq   %%mm1,(%1)\n\t"

	       : : "r"(sp) ,"r"(dp)
	       );
      
	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "movq  %%mm1,(%1)\n\t"

	       : : "r"(sp),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if (!half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "pavgb  (%2),%%mm1\n\t"
	       "movq  %%mm1,(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "movq   (%1),%%mm2\n\t"
	       "pavgb 1(%1),%%mm2\n\t"
	       "pavgb %%mm1,%%mm2\n\t"
	       "pavgb  (%2),%%mm2\n\t"
	       "movq  %%mm2,(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
  }



  void MotionCompensation_MMX2_16::Compensate(Pixel* dest,int dest_stride,
					      const Bitmap<Pixel>& refbm,
					      int ref_x0,int ref_y0,
					      const MotionVector& mv,int blkh)
  {
    int mv_h = mv.h>>16;
    int mv_v = mv.v>>16;

    bool half_h = (mv.h & 0x8000);
    bool half_v = (mv.v & 0x8000);

    const Pixel*const* ref = refbm.AskFrame();

    int  src_stride = refbm.AskStride();
    const Pixel* sp = &ref[ref_y0+mv_v][ref_x0+mv_h];
    Pixel* dp = dest;

    if (!half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    const uint64* spd = (const uint64*)sp;
	    uint64* dpd = (uint64*)dp;

	    dpd[ 0]= spd[ 0];
	    dpd[ 1]= spd[ 1];

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "movq  %%mm1,(%1)\n\t"

	       "movq  8(%0),%%mm2\n\t"
	       "pavgb 9(%0),%%mm2\n\t"
	       "movq  %%mm2,8(%1)\n\t"

	       : : "r"(sp),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if (!half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "movq  %%mm1,(%2)\n\t"

	       "movq  8(%0),%%mm2\n\t"
	       "pavgb 8(%1),%%mm2\n\t"
	       "movq  %%mm2,8(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "movq   (%1),%%mm2\n\t"
	       "pavgb 1(%1),%%mm2\n\t"
	       "pavgb %%mm1,%%mm2\n\t"
	       "movq  %%mm2,(%2)\n\t"

	       "movq  8(%0),%%mm1\n\t"
	       "pavgb 9(%0),%%mm1\n\t"
	       "movq  8(%1),%%mm2\n\t"
	       "pavgb 9(%1),%%mm2\n\t"
	       "pavgb %%mm1,%%mm2\n\t"
	       "movq  %%mm2,8(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
  }


  void MotionCompensation_MMX2_16avg::Compensate(Pixel* dest,int dest_stride,
						 const Bitmap<Pixel>& refbm,
						 int ref_x0,int ref_y0,
						 const MotionVector& mv,int blkh)
  {
    int mv_h = mv.h>>16;
    int mv_v = mv.v>>16;

    bool half_h = (mv.h & 0x8000);
    bool half_v = (mv.v & 0x8000);

    const Pixel*const* ref = refbm.AskFrame();

    int  src_stride = refbm.AskStride();
    const Pixel* sp = &ref[ref_y0+mv_v][ref_x0+mv_h];
    Pixel* dp = dest;

    if (!half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "movq   %%mm1,(%1)\n\t"

	       "movq   8(%0),%%mm1\n\t"
	       "pavgb  8(%1),%%mm1\n\t"
	       "movq   %%mm1,8(%1)\n\t"

	       : : "r"(sp) ,"r"(dp)
	       );
      
	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h && !half_v)
      {
	for (int y=0;y<blkh;y++)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "movq  %%mm1,(%1)\n\t"

	       "movq  8(%0),%%mm2\n\t"
	       "pavgb 9(%0),%%mm2\n\t"
	       "pavgb 8(%1),%%mm2\n\t"
	       "movq  %%mm2,8(%1)\n\t"

	       : : "r"(sp),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if (!half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb  (%1),%%mm1\n\t"
	       "pavgb  (%2),%%mm1\n\t"
	       "movq  %%mm1,(%2)\n\t"

	       "movq  8(%0),%%mm2\n\t"
	       "pavgb 8(%1),%%mm2\n\t"
	       "pavgb 8(%2),%%mm2\n\t"
	       "movq  %%mm2,8(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
    else if ( half_h &&  half_v)
      {
	for (int y=0;y<blkh;y++,dest += dest_stride)
	  {
	    __asm__
	      (
	       "movq   (%0),%%mm1\n\t"
	       "pavgb 1(%0),%%mm1\n\t"
	       "movq   (%1),%%mm2\n\t"
	       "pavgb 1(%1),%%mm2\n\t"
	       "pavgb %%mm1,%%mm2\n\t"
	       "pavgb  (%2),%%mm2\n\t"
	       "movq  %%mm2,(%2)\n\t"

	       "movq  8(%0),%%mm1\n\t"
	       "pavgb 9(%0),%%mm1\n\t"
	       "movq  8(%1),%%mm2\n\t"
	       "pavgb 9(%1),%%mm2\n\t"
	       "pavgb %%mm1,%%mm2\n\t"
	       "pavgb 8(%2),%%mm2\n\t"
	       "movq  %%mm2,8(%2)\n\t"

	       : : "r"(sp),"r"(sp+src_stride),"r"(dp)
	       );

	    dp += dest_stride;
	    sp += src_stride;
	  }
      }
  }

}
