/********************************************************************************
    LibVideoCoding - video coding library
    Copyright (C) 2002  Dirk Farin

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public License
    as published by the Free Software Foundation; either version 2
    of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 ********************************************************************************/

#include "libvideocoding/motioncomp/motioncomp_scalar.hh"

namespace videocoding {

  MotionCompensation_Scalar::MotionCompensation_Scalar(int blkw,bool avg)
    : d_blkw(blkw), d_avg(avg)
  {
  }


  void MotionCompensation_Scalar::Compensate(Pixel* dest,int dest_stride,
					     const Bitmap<Pixel>& refbm,
					     int ref_x0,int ref_y0,
					     const MotionVector& mv,int blkh)
  {
    int mv_h = mv.h>>16;
    int mv_v = mv.v>>16;

    bool half_h = (mv.h & 0x8000);
    bool half_v = (mv.v & 0x8000);

    const Pixel*const* ref = refbm.AskFrame();

    if (!half_h && !half_v)
      {
	if (!d_avg)
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		dest[x] = ref[ref_y0+y+mv_v][ref_x0+x+mv_h];
	  }
	else
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		dest[x] = (dest[x] + ref[ref_y0+y+mv_v][ref_x0+x+mv_h] +1)/2;
	  }
      }
    else if ( half_h && !half_v)
      {
	if (!d_avg)
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		dest[x] = (ref[ref_y0+y+mv_v][ref_x0+x+mv_h]+
			   ref[ref_y0+y+mv_v][ref_x0+x+mv_h+1]+1)/2;
	  }
	else
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		dest[x] = (dest[x] + (ref[ref_y0+y+mv_v][ref_x0+x+mv_h]+
				      ref[ref_y0+y+mv_v][ref_x0+x+mv_h+1]+1)/2  +1)/2;
	  }
      }
    else if (!half_h &&  half_v)
      {
	if (!d_avg)
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		dest[x] = (ref[ref_y0+y+mv_v  ][ref_x0+x+mv_h]+
			   ref[ref_y0+y+mv_v+1][ref_x0+x+mv_h]+1)/2;
	  }
	else
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		dest[x] = (dest[x] + (ref[ref_y0+y+mv_v  ][ref_x0+x+mv_h]+
				      ref[ref_y0+y+mv_v+1][ref_x0+x+mv_h]+1)/2  +1)/2;
	  }
      }
    else if ( half_h &&  half_v)
      {
	if (!d_avg)
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		{
		  dest[x] = (ref[ref_y0+y+mv_v  ][ref_x0+x+mv_h  ]+
			     ref[ref_y0+y+mv_v  ][ref_x0+x+mv_h+1]+
			     ref[ref_y0+y+mv_v+1][ref_x0+x+mv_h  ]+
			     ref[ref_y0+y+mv_v+1][ref_x0+x+mv_h+1]+2)/4;
		}
	  }
	else
	  {
	    for (int y=0;y<blkh;y++,dest += dest_stride)
	      for (int x=0;x<d_blkw;x++)
		{
		  dest[x] = (dest[x] + (ref[ref_y0+y+mv_v  ][ref_x0+x+mv_h  ]+
					ref[ref_y0+y+mv_v  ][ref_x0+x+mv_h+1]+
					ref[ref_y0+y+mv_v+1][ref_x0+x+mv_h  ]+
					ref[ref_y0+y+mv_v+1][ref_x0+x+mv_h+1]+2)/4   +1)/2;
		}
	  }
      }
  }

}

