/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/decoder/motioncomp.hh,v 1.4 2002/05/12 21:41:47 farin Exp $

    Motion compensation for MPEG-2 motion prediction.
 ********************************************************************************
    Copyright (C) 2002 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_DECODER_MOTIONCOMP_HH
#define LIBVIDEOCODING_MPEG2_DECODER_MOTIONCOMP_HH

#include <libvideogfx/graphics/datatypes/image.hh>
#include <libvideocoding/mpeg2/syntax/macroblock.hh>
#include <libvideocoding/motioncomp/motioncomp.hh>


namespace videocoding {
  using namespace videogfx;

  class MP2_MotionCompensation
  {
  public:
    MP2_MotionCompensation(bool accurate=false);
    ~MP2_MotionCompensation();

    void InitializeReferences(const Image<Pixel>* last,
			      const Image<Pixel>* next,
			      const Image<Pixel>* curr,
			      bool  P_picture=true,
			      bool  field_structure=false,
			      bool  first_field=true,
			      bool  predicting_top_field=true);

    /*
      void SetForwardReferenceFrame (const Image<Pixel>& img)
      { SetForwardReferences(img.AskBitmapY(),img.AskBitmapCb(),img.AskBitmapCr()); }
      void SetBackwardReferenceFrame(const Image<Pixel>& img)
      { SetBackwardReferences(img.AskBitmapY(),img.AskBitmapCb(),img.AskBitmapCr()); }

      void SetForwardReferences (const Bitmap<Pixel>& bmy,
      const Bitmap<Pixel>& bmcb,
      const Bitmap<Pixel>& bmcr, bool may_use_fields = true);
      void SetBackwardReferences(const Bitmap<Pixel>& bmy,
      const Bitmap<Pixel>& bmcb,
      const Bitmap<Pixel>& bmcr, bool may_use_fields = true);
    */

    void CreatePrediction(const MP2_Macroblock&,
			  int x0,int y0,
			  Pixel* yp,int y_stride,
			  Pixel* up,int u_stride,
			  Pixel* vp,int v_stride);

  private:
    Bitmap<Pixel> d_ref_ybm [2][3];  /* [ fwd/bkw ][ frm/top/bot ] */
    Bitmap<Pixel> d_ref_cbbm[2][3];
    Bitmap<Pixel> d_ref_crbm[2][3];

    MotionCompensation* d_motioncomp_16;
    MotionCompensation* d_motioncomp_8;
    MotionCompensation* d_motioncomp_16_avg;
    MotionCompensation* d_motioncomp_8_avg;

    void CompensateAll(const MP2_Macroblock& mb,
		       Bitmap<Pixel>& ref_y,
		       Bitmap<Pixel>& ref_cb,
		       Bitmap<Pixel>& ref_cr,
		       int y0,int x0,
		       MotionVector mv,
		       Pixel* bm_y,  int bm_y_stride,
		       Pixel* bm_cb, int bm_cb_stride,
		       Pixel* bm_cr, int bm_cr_stride,
		       int blkh,
		       bool add);
  };

}

#endif
