/********************************************************************************
  $Header: /home/farin/cvs/libvideocoding/libvideocoding/mpeg2/syntax/macroblock.hh,v 1.7 2002/05/12 21:41:47 farin Exp $

    MPEG-2 macroblock definition.
 ********************************************************************************
    Copyright (C) 2002 Dirk Farin (see the README for complete list of authors)

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_SYNTAX_MACROBLOCK_HH
#define LIBVIDEOCODING_MPEG2_SYNTAX_MACROBLOCK_HH

#include <libvideocoding/datatypes/dct.hh>
#include <libvideocoding/datatypes/mv.hh>
#include <libvideocoding/mpeg2/syntax/video.hh>
#include <libvideocoding/mpeg2/syntax/dct.hh>
#include <libvideocoding/mpeg2/syntax/sliceconf.hh>
#include <libvideogfx/utility/bitstream/bitreader_fast.hh>


namespace videocoding {
  using namespace videogfx;

  enum MP2_PredictionField
    {
      MP2_Pred_TopField=0,
      MP2_Pred_BottomField=1
    };


  struct MP2_Motion
  {
    MP2_Prediction_Mode prediction_mode;

    bool has_mv_forward;
    bool has_mv_backward;

    // r - number of vector
    // s - temporal direction (0: forward / 1: backward)

    MotionVector        vector               [2 /*r*/][2 /*s*/];
    MP2_PredictionField vertical_field_select[2 /*r*/][2 /*s*/];
    int2                dmvector             [2 /*r*/][2 /*s*/][2];

    bool dmv;
  };


  struct MP2_Macroblock
  {
    // MP2_Macroblock();

    const MP2_SliceHeader* slicehdr;

    bool skipped; /* This is a skipped macroblock (this is only used for informational purpose
		     when reading macroblocks). */

    // --- DCT ---

    bool use_field_DCT;

    uint7  qcode; /* 1 to 31  */

    /* to ensure backward-compatibility */
    //#define quantizer_scale_code qcode

    DCTBlock_8x8 dct_y [2][2];
    DCTBlock_8x8 dct_cb[2][2];
    DCTBlock_8x8 dct_cr[2][2];


    // --- motion compensation ---

    MP2_Motion motion;


    DCTBlock_8x8&       AskBlock(int nr);
    const DCTBlock_8x8& AskBlock(int nr) const;

    void CopyAllExceptDCTCoeff(const MP2_Macroblock&);
  };


  void Debug_ShowMB(const MP2_Macroblock&);


  // ----------------------------------------------------------------


  class MP2_MacroblockEncoder
  {
  public:
    MP2_MacroblockEncoder();

    /* initialization */

    void SetOutputStream(class BitBuffer& bb) { d_bs=&bb; }
    void SetSliceConfiguration(const MP2_SliceConfiguration* sliceconf) { d_sliceconf = sliceconf; }

    /* usage */

    void Write(const MP2_Macroblock&,bool last_in_slice); /* slice configuration not needed */
    void Write(const MP2_Macroblock&,int y,int x); /* slice configuration required */

  private:
    class BitBuffer* d_bs;
    const MP2_SliceConfiguration* d_sliceconf;

    bool d_first_in_slice;
    int  d_MBincr;

    int d_qscale;   // current qscale code

    int d_pred_y;   // current DC predictors
    int d_pred_cb;
    int d_pred_cr;

    MotionVector d_PMV[2][2]; // [r][s]

    int d_last_qcode;

    bool d_lastmb_had_mv_forward;
    bool d_lastmb_had_mv_backward;
    bool d_lastmb_was_intra;

    void ResetPMV();
    void ResetDCPred(const MP2_PictureHeader*);

    void CodeMotionCode(int diff,int f,int rsize);
    void motion_vectors(const MP2_Macroblock& mb,int direction);
    void motion_vector (const MP2_Macroblock& mb,int n,int direction);
  };


  // ----------------------------------------------------------------

  class MP2_MacroblockSink
  {
  public:
    virtual ~MP2_MacroblockSink() { }

    virtual MP2_Macroblock* RequestMacroblock(int mbx,int mby) = 0;
    virtual void SendMacroblock(MP2_Macroblock*,int mbx,int mby) { }
  };


  class MP2_MacroblockDecoder
  {
  public:
    MP2_MacroblockDecoder();
    ~MP2_MacroblockDecoder();

    /* initialization */

    void SetMBSink(MP2_MacroblockSink&);
    void EnableOnTheFlyDequantization(bool flag=true) { d_dctdecoder.EnableOnTheFlyDequantization(flag); }
    void Reset();

    /* configuration */

    void SetSequenceHeader(const MP2_SequenceHeader&);
    void SetPictureHeader(const MP2_PictureHeader&);
    void SetSliceHeader(const MP2_SliceHeader&);

    /* usage */

    void Decode(BitReader_Fast&);

  private:
    MP2_MacroblockSink* d_sink;

    MP2_DCTBlockDecoder d_dctdecoder;

    const MP2_SequenceHeader* d_seqhdr;
    const MP2_PictureHeader*  d_pichdr;
    const MP2_SliceHeader*    d_slicehdr;

    MP2_Motion d_PMV;   // NOTE: data is NOT in fixed16_16 format but as simple integers instead !

    int d_dcpred_y;
    int d_dcpred_cb;
    int d_dcpred_cr;

    int d_qcode;

    void ResetDCPredictors();
    void ResetMotionPredictors();

    void motion_vectors(struct MP2_Motion&,int s, class BitReader_Fast&);
    void motion_vector (struct MP2_Motion&,int r, int s, class BitReader_Fast&);
  };

}

#endif
