/********************************************************************************
  libvideocoding/mpeg2/syntax/scsplitter.hh
    start code splitter

  purpose:
    Splits PES packets into new packets so that the new
    packets begin with the specified start codes. You may
    split at a sub-set of start codes.
    PTS and DTS values are transfered to the new packets
    if they refer to their content.

  notes:

  to do:

  author(s):
   - Dirk Farin, dirk.farin@gmx.de

  modifications:
   03/Mar/2002 - Dirk Farin
     - generalized videoauconv.hh into scsplitter.hh
   22/Oct/1999 - Dirk Farin
     - If stream does not start with 000001, data until first startcode is
       skipped.
   27/Dec/1998 - Dirk Farin
     - Bugfix: very first start code in file was not found if it
       had more leading 0-bytes than needed.
   22/Dec/1998 - Dirk Farin
     - Bugfix: the last byte of a input packet was inserted into
       the destination packet twice.
   19/Dec/1998 - Dirk Farin
     - first implementation, PTS insertion when undefined is not
       implemented yet.
 ********************************************************************************
    Copyright (C) 2002  Dirk Farin

    This program is distributed under GNU Public License (GPL) as
    outlined in the COPYING file that comes with the source distribution.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 ********************************************************************************/

#ifndef LIBVIDEOCODING_MPEG2_SYNTAX_SCSPLITTER_HH
#define LIBVIDEOCODING_MPEG2_SYNTAX_SCSPLITTER_HH

#include <libvideocoding/mpeg2/syntax/pessource.hh>

namespace videocoding {
  using namespace videogfx;

  class MP2_StartCodeSplitter : public MP2_PES_Source
  {
  public:
    MP2_StartCodeSplitter();
    ~MP2_StartCodeSplitter();

    /* initialization */

    void SetSource(MP2_PES_Source& src);
    void Reset();

    void SetBreakpoint(MP2_StartCode sc,bool flag=true) { d_breakpoint[sc]=flag; }
    void SetBreakpointAllSC(bool flag=true);


    /* usage */

    MP2_SystemPacket* GetNextPacket();
    bool              IsEOF() const;

  private:
    MP2_PES_Source* d_src;

    MP2_PES_Packet* d_current;
    MP2_PES_Packet* d_next;

    int    d_curr_pos;
    int    d_next_overhang;

    uint8  d_breakpoint[256];

    //ByteBufferParams d_bytebufparams;

    void AddLookahead();
  };

}

#endif
